#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <stdint.h>
#include <event.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "sap.h"
#include "getstream.h"
#include "dvbrc.h"

#if 0
#define SAP_V4_GLOBAL_ADDRESS   "224.2.127.254"
/* Organization-local SAP address */
#define SAP_V4_ORG_ADDRESS      "239.195.255.255"
/* Local (smallest non-link-local scope) SAP address */
#define SAP_V4_LOCAL_ADDRESS    "239.255.255.255"
/* Link-local SAP address */
#define SAP_V4_LINK_ADDRESS     "224.0.0.255"
#endif

#define SAP_ADDR	"224.2.127.254"			/* Scope Global */
#define SAP_PORT	9875				/* As per RFC 2974 */
#define SAP_MAX_SIZE	1024				/* As per RFC 2974 */

/*
 * Implement Mini-SAP Service
 *
 * RFC 2974 (Session Announcement Protocol)
 * http://www.ietf.org/rfc/rfc2974.txt
 * RFC 2327 (SDP: Session Description Protocol)
 * http://www.ietf.org/rfc/rfc2327.txt
 *
 */

static int init_sap_timer(void );
static int sapsock;

struct sappkt_s {
	uint8_t 	version:3,
			     addrtype:1,
			     reserved:1,
			     msgtype:1,
			     encryption:1,
			     compressed:1;
	uint8_t		authlen;
	uint16_t	msgidhash;
	uint32_t	origin;
};

#define SAP_ADDRTYPE_V4		0
#define SAP_ADDRTYPE_V6		1

#define SAP_MSGTYPE_ANNOUNCE	1
#define SAP_MSGTYPE_DELETE	0

static uint8_t	sappkt[SAP_MAX_SIZE];

static void send_sap(int fd, short event, void *arg) {
	struct stream		*s=streams;
	unsigned char		*sp;
	uint32_t		addr=inet_addr(SAP_ADDR);
	char			name[128];

	while (s) {
		if (s->nuccs >= 0) { /* Do not announce unicast streams */
			s=s->next;
			continue;
		}
		memset(&sappkt, 0, SAP_MAX_SIZE);
#define SAP_VERSION		1
#define SAP_VERSION_SHIFT	5

		sappkt[0]=SAP_VERSION<<SAP_VERSION_SHIFT;	/* Version + Bitfield */
		sappkt[1]=0x0;					/* Auth len */
		sappkt[2]=s->channel->sid>>8&0xff;
		sappkt[3]=s->channel->sid&0xff;

		sappkt[4]=addr&0xff;
		sappkt[5]=addr>>8&0xff;
		sappkt[6]=addr>>16&0xff;
		sappkt[7]=addr>>24&0xff;

		sp=sappkt+8;

		sp+=sprintf(sp, "v=0\r\n");
		sp+=sprintf(sp, "o=- %x 1 IN IP4 %s\r\n", s->channel->sid, s->ipaddr);
		sprintf(name, "%s %s", (s->vpid == 0) ? "Radio" : "TV", s->channel->name);
		sp+=sprintf(sp, "s=%s\r\n", name);
		sp+=sprintf(sp, "t=0 0\r\n");
		sp+=sprintf(sp, "c=IN IP4 %s/%d\r\n", s->ipaddr, s->ttl);
		/* RFC 1890 RTP Audio/Video Profile - MP2T - Mpeg 2 Transport Stream */
		sp+=sprintf(sp, "m=%s %d udp 33\r\n", (s->vpid == 0) ? "audio" : "video", s->port);
		sp+=sprintf(sp, "a=tool:getstream\r\n");
		sp+=sprintf(sp, "a=type:broadcast\r\n");

		send(sapsock, sappkt, sp-sappkt, MSG_DONTWAIT);

		s=s->next;
	}

	init_sap_timer();
}


static int init_sap_timer(void ) {
	static struct event	sapevent;
	static struct timeval	tv;

	tv.tv_usec=0;
	tv.tv_sec=1;

	evtimer_set(&sapevent, send_sap, &sapevent);
	evtimer_add(&sapevent, &tv);

	return 1;
}

static int init_sap_sock(void ) {
	struct sockaddr_in	sin;
	unsigned int		ttl;

	sapsock=socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if (sapsock == -1) {
		printf("Failed to open sap socket\n");
		exit(-1);
	}

	sin.sin_family=AF_INET;
	inet_aton(SAP_ADDR, &sin.sin_addr);
	sin.sin_port=htons(SAP_PORT);

	/* FIXME - error checking */
	connect(sapsock, (struct sockaddr *) &sin, sizeof(struct sockaddr_in));

	ttl=SAP_DEFAULT_TTL;
	setsockopt(sapsock, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));

	return 1;
}


int init_sap(void ) {
	init_sap_sock();
	init_sap_timer();

	return 1;
}
